"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const buffer_reader_1 = __importDefault(require("buffer-reader"));
const fs = __importStar(require("fs"));
require("../extensions/string");
const data_types_1 = __importDefault(require("./data-types"));
class SauceParser {
    parse(filePath) {
        this.validateFile(filePath);
        const fullFileSize = fs.statSync(filePath).size;
        const fd = fs.openSync(filePath, 'r');
        const buffer = Buffer.alloc(fullFileSize);
        fs.readSync(fd, buffer, 0, fullFileSize, 0);
        const reader = new buffer_reader_1.default(buffer);
        if (fullFileSize < SauceParser.SAUCE_SIZE) {
            return null;
        }
        reader.seek(fullFileSize - SauceParser.SAUCE_SIZE);
        const id = reader.nextString(5);
        if (id !== SauceParser.SAUCE_ID) {
            return null;
        }
        const version = reader.nextString(2);
        const title = reader.nextString(35).trim();
        const author = reader.nextString(20).trim();
        const group = reader.nextString(20).trim();
        const date = this.parseDate(reader.nextString(8));
        const fileSize = reader.nextUInt32LE();
        const dataType = data_types_1.default(reader.nextUInt8());
        const fileType = reader.nextUInt8();
        const tInfo1 = reader.nextUInt16LE();
        const tInfo2 = reader.nextUInt16LE();
        const tInfo3 = reader.nextUInt16LE();
        const tInfo4 = reader.nextUInt16LE();
        const commentLines = reader.nextUInt8();
        const flags = reader.nextUInt8();
        const tInfoS = reader.nextString(22).strip('\u0000');
        const comments = commentLines > 0 ?
            this.parseComments(reader, fullFileSize, commentLines) : [];
        return {
            id, version, title, group, author, date, fileSize, dataType,
            fileType, tInfo1, tInfo2, tInfo3, tInfo4, comments, flags, tInfoS
        };
    }
    validateFile(filePath) {
        if (!fs.existsSync(filePath)) {
            throw new Error(`File ${filePath} not found`);
        }
    }
    parseDate(date) {
        const y = Number(date.substr(0, 4));
        const m = Number(date.substr(4, 2)) - 1;
        const d = Number(date.substr(6, 2));
        return new Date(y, m, d);
    }
    parseComments(reader, fullFileSize, commentLines) {
        reader.seek(this.commentPosition(fullFileSize, commentLines));
        const commentId = reader.nextString(SauceParser.COMMENT_ID_SIZE);
        const result = [];
        if (commentId === SauceParser.COMMENT_ID) {
            for (let i = 0; i < commentLines; i++) {
                const line = reader.nextString(SauceParser.COMMENT_SIZE).trim();
                result.push(line);
            }
        }
        return result;
    }
    commentPosition(fullFileSize, commentLines) {
        const commentsSize = SauceParser.COMMENT_ID_SIZE + commentLines * SauceParser.COMMENT_SIZE;
        return fullFileSize - SauceParser.SAUCE_SIZE - commentsSize;
    }
}
SauceParser.SAUCE_ID = 'SAUCE';
SauceParser.COMMENT_ID = 'COMNT';
SauceParser.SAUCE_SIZE = 128;
SauceParser.COMMENT_SIZE = 64;
SauceParser.COMMENT_ID_SIZE = 5;
exports.default = SauceParser;
